package ch.psi.daq.queryrest.response.csv;

import java.util.function.Function;

import ch.psi.daq.common.util.Arrays;
import ch.psi.daq.domain.DataEvent;
import ch.psi.data.collection.PrimitiveList;

public class QueryFieldStringifyer implements Function<DataEvent, String> {
   public static final String OPEN_BRACKET = "[";
   public static final String CLOSE_BRACKET = "]";

   private Function<DataEvent, Object> accessor;
   private String nonValue;
   private String arraySeparator;

   public QueryFieldStringifyer(Function<DataEvent, Object> accessor, String nonValue, String arraySeparator) {
      this.accessor = accessor;
      this.nonValue = nonValue;
      this.arraySeparator = arraySeparator;
   }

   @SuppressWarnings("rawtypes")
   @Override
   public String apply(DataEvent event) {
      if (event == null) {
         return nonValue;
      }

      Object value = accessor.apply(event);
      if (value == null) {
         return nonValue;
      } else if (value instanceof PrimitiveList) {
         return toString((PrimitiveList) value, arraySeparator);
      } else if (value.getClass().isArray()) {
         return Arrays.toString(value, arraySeparator);
      } else {
         return value.toString();
      }
   }

   public static String toString(PrimitiveList<?> value, String arraySeparator) {
      int size = value.size();
      if (size < 1) {
         return Arrays.NULL_STRING;
      } else if (size == 1) {
         // scalar
         return value.getAsString(0);
      } else {
         int iMax = size - 1;
         if (iMax == -1)
            return Arrays.EMPTY_ARRAY;

         StringBuilder b = new StringBuilder();
         b.append(OPEN_BRACKET);
         for (int i = 0;; i++) {
            b.append(value.getAsString(i));
            if (i == iMax)
               return b.append(CLOSE_BRACKET).toString();
            b.append(arraySeparator);
         }
      }
   }
}
