package ch.psi.daq.test.queryrest.query;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.psi.daq.cassandra.reader.CassandraReader;
import ch.psi.daq.common.time.TimeUtils;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.events.ChannelConfiguration;
import ch.psi.daq.domain.events.MetaPulseId;
import ch.psi.daq.domain.events.impl.MetaPulseIdImpl;
import ch.psi.daq.domain.query.range.PulseIdRangeQuery;
import ch.psi.daq.domain.query.range.TimeRangeQuery;

public class DummyCassandraReader extends AbstractStreamEventReader implements CassandraReader {
   private static final Logger LOGGER = LoggerFactory.getLogger(DummyCassandraReader.class);
   
   public DummyCassandraReader() {
   }

   @PostConstruct
   public void afterPropertiesSet() {
      init(Backend.SF_DATABUFFER);
   }

   @Override
   public ChannelConfiguration getChannelConfigurationBefore(TimeRangeQuery query) {
      try {
         return getChannelConfigurationBeforeAsync(query)
               .get(30, TimeUnit.SECONDS);
      } catch (Throwable t) {
         LOGGER.error("Could not read ChannelConfiguration from DB.", t);
         return null;
      }
   }

   @Override
   public CompletableFuture<ChannelConfiguration> getChannelConfigurationBeforeAsync(TimeRangeQuery query) {
      // implement when needed
      throw new UnsupportedOperationException();
   }

   @Override
   public ChannelConfiguration getChannelConfigurationAfter(TimeRangeQuery query) {
      try {
         return getChannelConfigurationAfterAsync(query)
               .get(30, TimeUnit.SECONDS);
      } catch (Throwable t) {
         LOGGER.error("Could not read ChannelConfiguration from DB.", t);
         return null;
      }
   }

   @Override
   public CompletableFuture<ChannelConfiguration> getChannelConfigurationAfterAsync(TimeRangeQuery query) {
      // implement when needed
      throw new UnsupportedOperationException();
   }
   
   @Override
   public CompletableFuture<MetaPulseId> getStartMetaPulseIdAsync(PulseIdRangeQuery query) {
      return CompletableFuture.completedFuture(new MetaPulseIdImpl(query.getChannel(), getBackend(), query.getStartPulseId(),
            TimeUtils.getTimeFromMillis(query.getStartPulseId() * 10, 0)));
   }

   @Override
   public CompletableFuture<MetaPulseId> getEndMetaPulseIdAsync(PulseIdRangeQuery query) {
      return CompletableFuture.completedFuture(new MetaPulseIdImpl(query.getChannel(), getBackend(), query.getEndPulseId(),
            TimeUtils.getTimeFromMillis(query.getEndPulseId() * 10, 0)));
   }
}
